import click
import os
import time
import sqlite3

from ..tex.path_tex import path_mechanics
from ..tex.path_tex import path_electrodynamics
from .database_functions import db_file
from .database_functions import create_connection
from .database_functions import createDatabase

time_date = int(time.strftime("%H%M%S%d%m%Y"))

def insertData(chapter, post_type, format_problem):
    database = create_connection(db_file(chapter, post_type, format_problem))

    cursor = database.cursor()

    if post_type == 'equation':
        execute_statement = f'INSERT INTO equation(chapter, date)VALUES("{chapter}", "{time_date}");'
        #print(execute_statement)
        cursor.execute(execute_statement)
        database.commit()
        database.close()

    elif post_type == 'problem':
        execute_statement = f'INSERT INTO problem(chapter, date) VALUES("{chapter}", "{time_date}");'
        #print(execute_statement)
        cursor.execute(execute_statement)
        database.commit()
        database.close()

    elif post_type == 'notes':
        execute_statement = f'INSERT INTO notes(chapter, date) VALUES("{chapter}", "{time_date}");'
        cursor.execute(execute_statement)
        database.commit()
        database.close()

    elif post_type == 'ideas':
        execute_statement = f'INSERT INTO ideas(chapter, date) VALUES("{chapter}", "{time_date}");'
        cursor.execute(execute_statement)
        database.commit()
        database.close()









import click
from .functions_tex import extract_tex_env


@click.command(
        help="Extracts tex environments from tex files"
        )
@click.option(
        '-i',
        '--inputfile',
        type=click.Path(),
        default="./main.tex",
        help="Input file path"
        )
@click.option(
        '-o',
        '--outputfile',
        type = click.Path(),
        default = "./tikz.tex",
        help = "Output file path"
        )
@click.option(
        '-e',
        '--environment',
        type=click.Choice(['tikzpicture', 'align*']),
        default="tikzpicture",
        help="Environment to be extracted"
        )
def extracttexenv(inputfile, outputfile, environment):
    extract_tex_env(inputfile, outputfile, environment)















import os
from rich import print

def print_equation(n, chapter, path):
    print(f'\n\tEquation-{n:02} of {chapter} is initiated at :\n\t{path}\n')




def print_problem(n, chapter, path):
    print(f'\n\tProblem-{n:02} of {chapter} is initiated at :\n\n\t{path}\n')




def print_notes(n, chapter, path):
    print(f'\tNotes-{n:02} of {chapter} is initiated at :\n\t{path}')




def print_ideas(n, chapter, path):
    print(f'\tIdea-{n:02} of {chapter} is initiated at :\n\t{path}')




def bat_file(file_path):
    bat_file_path = f'bat {file_path}'
    os.system(bat_file_path)




import click
import os
import time
from sqlite3 import Error

from ..tex.path_tex import path_chapter
from ..tex.path_tex import chapters
from .insert_data import insertData
from .get_data import getData
from .database_functions import db_file
from .database_functions import create_connection
from .database_functions import createDatabase
from .get_data import get_n_data


eqn_number_without_database = int(time.strftime("%H%M%S%d%m%Y"))

@click.command(
        help="Creates equation format tex file"
        )
@click.option(
        '-c',
        '--chapter',
        default='rotation',
        help="Chapter name",
        type=click.Choice(
            chapters,
            case_sensitive=False),
        show_default=True
        )
@click.option(
        '-n',
        '--number',
        default=3,
        type=click.INT,
        show_default=True
        )
@click.option(
        '-p',
        '--post_type',
        default='equation',
        type=click.Choice([
            'equation',
            'problem',
            'notes',
            'ideas'
            ]),
        help="Type of post ",
        show_default=True
        )
def fetchdata(chapter, post_type, number):
    try:
        data = get_n_data(chapter, post_type, number)
        if data != None:
            for d in data:
                print(d)
    except Error as e:
        print(e)




import os

path_parent = os.environ["TEX_PARENT_PATH"]
path_mechanics = f'{path_parent}/mechanics'
path_electrodynamics = f'{path_parent}/electrodynamics'
path_modern_physics = f'{path_parent}/modern-physics'
path_optics = f'{path_parent}/optics'


modules = [
        'mechanics',
        'electrodynamics',
        'modern-physics',
        'optics'
        ]


chapters_mechanics = [
        'kinematics',
        'projectile',
        'laws-of-motion',
        'work-energy-power',
        'circular-motion',
        'rotation',
        'center-of-mass',
        'gravitation',
        'simple-harmonic-motion',
        'elasticity',
        'fluid-mechanics',
        ]
chapters_electrodynamics = [
        'electrostatics',
        'current-electricity',
        'capacitors',
        'magnetics',
        'electromagnetic-induction',
        'alternating-current',
        'electromagnetic-waves'
        ]
chapters_optics = [
        'ray-optics',
        'wave-optics',
        ]
chapters_modern_physics = [
        'atomic-structure',
        'photoelectric-effect',
        'x-rays',
        'nuclear-physics',
        'semiconductors',
        'communiation-system'
        ]

post_types = [
        'equation',
        'problem',
        'notes',
        'ideas'
        ]


problem_type = [
        'mine',
        'books',
        'jee',
        'neet',
        'jee_advanced'
        ]



chapters = chapters_mechanics + chapters_electrodynamics + chapters_modern_physics + chapters_optics


def path_post(chapter_folder, post_type, format_problem):
    if post_type == post_types[0]:
        equation_folder = os.path.join(chapter_folder, post_type)
        os.makedirs(equation_folder, exist_ok=True)
        return equation_folder

    elif post_type == post_types[1]:
        problem_folder = os.path.join(chapter_folder, post_type)
        problem_folder_with_format_problem = os.path.join(problem_folder, format_problem)
        os.makedirs(problem_folder_with_format_problem, exist_ok=True)
        return problem_folder_with_format_problem

    elif post_type == post_types[2]:
        notes_folder = os.path.join(chapter_folder, post_type)
        os.makedirs(notes_folder, exist_ok=True)
        return notes_folder
    
    elif post_type == post_types[3]:
        ideas_folder = os.path.join(chapter_folder, post_type)
        os.makedirs(ideas_folder, exist_ok=True)
        return ideas_folder


def path_chapter(chapter, post_type, format_problem):
    """
    chapter -> name of the chapter
    post_type -> ['equation', 'problem', 'notes', 'db']
    format_problem -> ['JEE', 'IIT-JEE', 'NEET', 'BOOKS', 'MINE']
    """

    if chapter in chapters_mechanics:
        chapter_folder = os.path.join(path_mechanics, chapter)
        return path_post(chapter_folder, post_type, format_problem)

    elif chapter in chapters_electrodynamics:
        chapter_folder = os.path.join(path_electrodynamics, chapter)
        return path_post(chapter_folder, post_type, format_problem)

    elif chapter in chapters_modern_physics:
        chapter_folder = os.path.join(path_modern_physics, chapter)
        return path_post(chapter_folder, post_type, format_problem)

    elif chapter in chapters_optics:
        chapter_folder = os.path.join(path_optics, chapter)
        return path_post(chapter_folder, post_type, format_problem)








import click
import os
import time
import sqlite3
from sqlite3 import Error

from ..tex.path_tex import path_chapter
from ..tex.path_tex import path_mechanics
from ..tex.path_tex import path_electrodynamics
from .database_functions import db_file
from .database_functions import create_connection
from .database_functions import createDatabase

def getData(chapter, post_type, format_problem):

    try:

        database = create_connection(db_file(chapter, post_type, format_problem))
        cursor = database.cursor()

        if post_type == 'equation':
	        output = cursor.execute("""SELECT * FROM equation ORDER BY id DESC LIMIT 1;""")
	        return output.fetchmany(1)
	
        elif post_type == 'problem':
	        output = cursor.execute("""SELECT * FROM problem ORDER BY id DESC LIMIT 1;""")
	        return output.fetchmany(1)

        elif post_type == 'notes':
	        output = cursor.execute("""SELECT * FROM notes ORDER BY id DESC LIMIT 1;""")
	        return output.fetchmany(1)
	
        elif post_type == 'ideas':
	        output = cursor.execute("""SELECT * FROM ideas ORDER BY id DESC LIMIT 1;""")
	        return output.fetchmany(1)



        database.close()

    except:

        createDatabase(chapter, post_type, foramt_problem)



def get_n_data(chapter, post_type, n):
    try:

        database = create_connection(db_file(chapter, post_type))
        cursor = database.cursor()

        if post_type == 'equation':
            execute_statement = f'SELECT * FROM equation ORDER BY id DESC LIMIT {n};'
            output = cursor.execute(execute_statement)
            return output.fetchmany(n)

        elif post_type == 'problem':
            execute_statement = f'SELECT * FROM problem ORDER BY id DESC LIMIT {n};'
            output = cursor.execute(execute_statement)
            return output.fetchmany(n)

        elif post_type == 'notes':
            execute_statement = f'SELECT * FROM notes ORDER BY id DESC LIMIT {n};'
            output = cursor.execute(execute_statement)
            return output.fetchmany(n)

        elif post_type == 'ideas':
            execute_statement = f'SELECT * FROM ideas ORDER BY id DESC LIMIT {n};'
            output = cursor.execute(execute_statement)
            return output.fetchmany(n)



        database.close()

    except Error as e:
        print(e)












"""
    functions_tex.py

    extracts tex environment from a tex file

"""

def create_files(filepath, n, nf, extension):
	"""
	This function takes four inputs and returns tex filenames with proper path 
	
	    filepath -> path to be modified with filename
	    n -> nth of files to be produced
	    nf -> number of files to be produced
        extension -> file extension
	
	    Eg: filepath = "./test/main.tex" -> "./test/main-0.tex, main-1.tex" like that
	"""
	path = filepath.split('.')
	pathinit = filepath.split('/')
	if nf == 1:
	    if pathinit[0] == '.':
	        return '.' + path[len(path)-2] + '.' + extension
	    else:
	        return path[len(path)-2] + '.' + extension
	else:
	    if pathinit[0] == '.':
	        return '.' + path[len(path)-2] + '-' + str(n + 1) +  '.' + extension
	    else:
	        return path[len(path)-2] + '-' + str(n + 1) + '.' + extension
	






def extract_tex_env(inputfile, outputfile, environment):
    start = []
    end = []
    ntikz = []


    with open(inputfile, 'r') as tex:
        for (n, line) in enumerate(tex):
            if line == ("\\begin{" + environment + "}\n"):
                start.append(n)
                ntikz.append(n)
            if line == "\\end{" + environment + "}\n":
                end.append(n)
    print(len(ntikz))
    for i in range(len(ntikz)):
        if len(ntikz) == 1:
            outputtex = open(create_files(outputfile, i, len(ntikz), "tex"), 'w')
            with open(inputfile, 'r') as tex:
                for (n, line) in enumerate(tex):
                    if n >= start[i] and n <= end[i]:
                        print(line)
                        outputtex.write(line)
        else:
            outputtex = open(create_files(outputfile, i, len(ntikz), "tex"), 'w')
            with open(inputfile, 'r') as tex:
                for (n, line) in enumerate(tex):
                    if n >= start[i] and n <= end[i]:
                        print(line)
                        outputtex.write(line)









equation_preamble = r"""\documentclass{article}
\usepackage{v-vertical-equation}
\begin{document}
\end{document}"""



equation_vertical_preamble = r"""\documentclass{article}
\usepackage{v-vertical-equation}
\begin{document}
\end{document}"""

equation_square_preamble = r"""\documentclass{article}
\usepackage{v-square-equation}
\begin{document}
\end{document}"""



def equation_head(n):
    return f'{chapter}-{n}'



def equation_title(n):
    return f'{title}'





import click
import os
import time
import sqlite3

from ..tex.path_tex import path_chapter
from ..tex.path_tex import chapters

@click.command(help="Updates the database")
@click.option('-c', '--chapter', type=click.Choice(chapters, case_sensitive=False))
@click.option('-t', '--title')
@click.option('-e', '--equation')
@click.option('-n', '--equation_number')
@click.option('-T', '--tikz')
def updateequation(chapter, title, equation, tikz, equation_number):
    path_database = os.path.join(
            path_chapter(
                chapter.lower(),
                'equation'
                ),
            'equation.db'
            )
    database = sqlite3.connect(path_database)
    print(f'Databse {database} opened.\n')

    cursor = database.cursor()

    data_table= f'UPDATE equation SET  title= "{title}", equation = "{equation}", tikz = "{tikz}" WHERE id={equation_number};'
    cursor.execute(data_table)
    database.commit()
    database.close()


@click.command(help="Updates the database")
@click.option('-c', '--chapter', type=click.Choice(chapters, case_sensitive=False))
@click.option('-p', '--problem')
@click.option('-n', '--problem_number')
@click.option('-T', '--tikz')
def updateproblem(chapter, problem, options, tikz, problem_number):
    path_database = os.path.join(
            path_chapter(
                chapter.lower(),
                'problem'
                ),
            'problem.db'
            )
    database = sqlite3.connect(path_database)
    print(f'Databse {database} opened.\n')

    cursor = database.cursor()

    data_table= f'UPDATE problem SET  problem= "{problem}", options = "{options}", tikz = "{tikz}" WHERE id={problem_number};'
    cursor.execute(data_table)
    database.commit()
    database.close()



@click.command(help="Updates the database")
@click.option('-c', '--chapter', type=click.Choice(chapters, case_sensitive=False))
@click.option('-N', '--note')
@click.option('-n', '--notes_number')
def updatenotes(chapter, note, notes_number):
    path_database = os.path.join(
            path_chapter(
                chapter.lower(),
                'notes'
                ),
            'notes.db'
            )
    database = sqlite3.connect(path_database)
    print(f'Databse {database} opened.\n')

    cursor = database.cursor()
    data_table = f'UPDATE notes SET  note="{note}" WHERE id={notes_number};'
    print(data_table)
    cursor.execute(data_table)
    database.commit()
    database.close()



@click.command(help="Updates the database")
@click.option('-c', '--chapter', type=click.Choice(chapters, case_sensitive=False))
@click.option('-i', '--idea')
@click.option('-n', '--ideas_number')
def updateideas(chapter, idea, ideas_number):
    path_database = os.path.join(
            path_chapter(
                chapter.lower(),
                'ideas'
                ),
            'ideas.db'
            )
    database = sqlite3.connect(path_database)
    print(f'Databse {database} opened.\n')

    cursor = database.cursor()
    data_table= f'UPDATE ideas SET  idea="{idea}" WHERE id={ideas_number};'
    cursor.execute(data_table)
    database.commit()
    database.close()





import click
import os
import time

from .tex.path_tex import path_chapter
from .tex.path_tex import chapters
from .database.insert_data import insertData
from .database.get_data import getData
from .print_functions import print_problem
from .print_functions import bat_file
from .choice_option import ChoiceOption


eqn_number_without_database = int(time.strftime("%H%M%S%d%m%Y"))



size_square = f'\\geometry{{\npaperwidth=5in, \npaperheight=5in, \ntop=15mm, \nbottom=15mm, \nleft=10mm, \nright=10mm\n}}\n\n'
size_h_rectangle = f'\\geometry{{\npaperwidth=8in, \npaperheight=4.5in, \ntop=15mm, \nbottom=15mm, \nleft=10mm, \nright=10mm\n}}\n\n'
size_v_rectangle = f'\\geometry{{\npaperwidth=4.5in, \npaperheight=8in, \ntop=15mm, \nbottom=15mm, \nleft=10mm, \nright=10mm\n}}\n\n'

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.command(
        context_settings = CONTEXT_SETTINGS,
        help="Creates problem format tex file"
        )
@click.option(
        '-c',
        '--chapter',
        prompt='Chapters',
        type=click.Choice(
            chapters,
            case_sensitive=False),
        cls=ChoiceOption,
        help="Chapter name",
        )
@click.option(
        '-f',
        '--format_problem',
        prompt='Format',
        default=1,
        type=click.Choice(['JEE', 'IIT-JEE', 'NEET', 'BOOK', 'MINE']),
        cls=ChoiceOption,
        help='Format of the problem'
        )
@click.option(
        '-s',
        '--size',
        prompt='Size',
        default=1,
        type=click.Choice(['square', 'h-rectangle', 'v-rectangle']),
        cls=ChoiceOption,
        show_default=True,
        help="Size of the canvas"
        )
@click.option(
        '-n',
        '--problem_number',
        type=click.INT
        )
@click.option(
        '-a',
        '--append_to_database',
        is_flag=True,
        default=True,
        prompt=True,
        help="flag (-a turns-on) appends the equation to database"
        )
def main(chapter, format_problem, size, problem_number, append_to_database):
    if append_to_database:
        try:
            problem_number = getData(chapter, 'problem', format_problem)[0][0] + 1
        except:
            problem_number = 1
        insertData(chapter, 'problem', format_problem)
    else:
        problem_number = eqn_number_without_database

    path_problem= os.path.join(
            path_chapter(chapter.lower(), 'problem', format_problem),
            f'problem-{problem_number:02}'
            )
    

    os.makedirs(path_problem, exist_ok=True)
    main_tex = os.path.join(path_problem, 'main.tex')
    with open(main_tex, 'w') as file:
        file.write(f'\\documentclass{{article}}\n')
        file.write(f'\\usepackage{{v-problem}}\n')
        if size == 'square':
            file.write(size_square)
        elif size == 'h-rectangle':
            file.write(size_h_rectangle)
        elif size == 'v-rectangle':
            file.write(size_v_rectangle)

        file.write(f'\\begin{{document}}\n')
        file.write(f'{problem_number}\n')
        file.write(f'\\end{{document}}\n')

    print_problem(problem_number, chapter, main_tex)
    bat_file(main_tex)
    time.sleep(1)
    os.system(f'open -a texmaker {main_tex}')
    print('\n\topening texmaker ...\n')
    time.sleep(1)







import os
from sqlite3 import Error
import sqlite3
from ..tex.path_tex import path_chapter

def db_file(chapter, post_type, format_problem):
    chapter_path = path_chapter(chapter.lower(), post_type, format_problem)
    path_database = os.path.join(chapter_path, f'{post_type}_{format_problem.lower()}.db')
    return path_database


def create_connection(db_file):
    """ create a database connection to the SQLite database
        :param db_file: database file
        :return: Connection object or None
    """
    conn = None
    try:
        conn = sqlite3.connect(db_file)
    except Error as e:
        print(e)

    return conn



def createDatabase(chapter, post_type, format_problem):
    database = create_connection(db_file(chapter, post_type, format_problem))

    #cursor = database.cursor()
    if post_type == 'equation':
        database.execute(
            """ CREATE TABLE IF NOT EXISTS equation(
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                chapter TEXT NOT NULL,
                date TEXT,
                title TEXT,
                equation TEXT,
                tikz TEXT
            ); """
            )
    elif post_type == 'problem':
        database.execute(
            """ CREATE TABLE IF NOT EXISTS problem(
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                chapter TEXT NOT NULL,
                date TEXT,
                problem TEXT,
                options TEXT,
                tikz TEXT
            ); """
            )
    elif post_type == 'notes':
        database.execute(
            """ CREATE TABLE IF NOT EXISTS notes(
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                chapter TEXT NOT NULL,
                date TEXT,
                note TEXT
            ); """
            )
    elif post_type == 'ideas':
        database.execute(
            """ CREATE TABLE ideas(
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                chapter TEXT NOT NULL,
                date TEXT,
                idea TEXT
            ); """
            )


    database.close()


